%% clear
clc;
clear;
close all;
format compact;
format short g;

%% Set up FEM struct and pre-assemble
fem = getInitFem;
prob.fem.dof = getDof(fem);

prob.theta(1).fun = @(mu) mu(1);
prob.theta(2).fun = @(mu) mu(2);

prob = performPreAssembling(fem,prob);

%% Compute matrices for discrete inner porduct
[fem.const{2:2:end}] = deal(1);
[C,D] = assemble(fem,'Out',{'C','D'});

L2 = D;
L2 = (L2+L2')/2;

H1 = C+D;
H1 = (H1+H1')/2;

En = C;
En = (En+En')/2;

prob.fem.innProd.L2 = L2;
prob.fem.innProd.H1 = H1;
prob.fem.innProd.En = En;

POD_Norm = 'H1';
RB_Norm = 'H1';
mu_bar = [1;1];

prob.fem.innProd.refparam = mu_bar;
prob.fem.innProd.refalpha = getCoercivity(prob, mu_bar, RB_Norm);
prob.fem.innProd.refthetas =  zeros(length(prob.fem.pre.lhs),1);
for q=1:length(prob.fem.pre.lhs)
  prob.fem.innProd.refthetas(q) = prob.fem.pre.lhs(q).theta(mu_bar);
end

%% RB training using Greedy

% parameter space w.r.t. \mu_1
D = [.01 100];
logD = log10(D);

% number of snapshots n_train
n_train = 101;

% parameter training set
Xi_train = [logspace(logD(1),logD(2),n_train); ...
            ones(1,n_train)];

% initial parameter
S_N0 = [min(D); 1];

%Greedy training
prob.RB = trainRB_Greedy(prob, Xi_train, RB_Norm, S_N0);

figure(1)
semilogy(prob.RB.Greedy_errors, 'b-x');
title('Greedy Error during training')
xlabel('N');
ylabel('$\max\limits_{\mu \in \Xi_{\mathrm{train}}} \Delta_N (\mu)$', 'Interpreter','LaTex')

%% RB tests

n_test = 40;
Xi_test = [10.^(sort(rand(1,n_test)*diff(logD)+logD(1))); ...
           ones(1,n_test)];

% compute RB errors and error estimates
err_RB = zeros(n_test,prob.RB.Nmax);
err_est = zeros(n_test,prob.RB.Nmax);

innProd = prob.fem.innProd.(prob.RB.Norm);
for n=1:n_test
  mu = Xi_test(:,n);
  
  u = getSol(prob, mu);
  
  for l = 1:prob.RB.Nmax

    [u_RB_coeffs,u_RB] = getRBAppr(prob.RB,mu,l);
    
    e = u-u_RB;
    err_RB(n,l) = sqrt(e'*innProd*e);
    err_est(n,l) = getErrEst_decomp(prob.RB, mu, u_RB_coeffs);
  end
  
end

figure(2);
loglog(Xi_test(1,:),err_RB,'-x');
hold on
loglog(Xi_test(1,:), err_est, '--o');
legend(num2str((1:prob.RB.Nmax)','N=%d'));
title('Error (-x) and error bound (--o) over test set');
xlabel('\mu')
ylabel('$\Vert u^{\mathcal N}(\mu) - u_N(\mu)\Vert$ bzw. $\Delta_N(\mu)$', 'Interpreter', 'LaTex')
hold off

[max_err_RB, ind] = max(err_RB,[],1);
figure(3)
semilogy(max_err_RB, '-x');
hold on
semilogy(diag(err_est(ind, :)), '--o');
title('Maximum Error and corresponding error bound over test set');
xlabel('N')
ylabel('$\max\limits_{\mu \in \Xi_{\mathrm{test}}} \Vert u^{\mathcal N}(\mu) - u_N(\mu)\Vert$ bzw. $\max\limits_{\mu \in \Xi_{\mathrm{test}}} \Delta_N (\mu)$', 'Interpreter', 'LaTex')
legend('Maximum Error', 'Error Bound')
hold off

